from datetime import datetime
import requests
import json
from voxyl.playerstats import PlayerStats
from voxyl.utils import *
from .constants import *

class GuildLB:
    def __init__(self, guildLBInfo, num, key=""):
        self.guildLB = guildLBInfo
        self.num = num
        self.key = key

    def get_guild(self, tag):
        for i in self.guildLB:
            if i["tag"].lower() == tag:
                guildInfo = requests.get(f"{VOXYLBASE}{VOXYLGUILDINFO.format(tag, VOXYLAPI.format(self.key))}").json()
                guildMembers = requests.get(f"{VOXYLBASE}{VOXYLGUILDMEMBERS.format(tag, VOXYLAPI.format(self.key))}").json()

                if not guildInfo["success"] or not guildMembers["success"]:
                    return None

                return GuildLBGuild(infoData=guildInfo, membersData=guildMembers["members"], placing=i["placing"], key=self.key)
        
        raise GuildNotFound(f"'{tag}' not found in top {self.num} requested guilds")
    

class GuildStats:
    def __init__(self, infoData, membersData, key=""):
        self.id = infoData["id"]
        self.name = infoData["name"]
        self.desc = infoData["desc"]
        self.xp = infoData["xp"]
        self.num = infoData["num"]
        self.owner = infoData["ownerUUID"]
        self.creationTimeRaw = infoData["time"]
        self.creationTime = datetime.fromtimestamp(self.creationTimeRaw).strftime("%I:%M %p on %B %d, %Y")
        self.allMembers = get_members(membersData)
        self.memberUUIDs = self.allMembers["owner"] + self.allMembers["admin"] + self.allMembers["moderator"] + self.allMembers["member"]
        self.rawData = membersData
        self.key = key

    def getGuildMember(self, ign):
        member = getUserInfo.formatUUID(getUserInfo.getUUID(ign))

        for i in self.rawData:
            if i["uuid"] == member:
                return GuildMember(i["uuid"], i["role"], i["time"], self.key)

        raise GuildMemberNotFound(f"{ign} not found in guild '{self.name}'")

    def getGuildMembers(self):
        self.membersList = []
        for i in self.rawData:
            self.membersList.append(GuildMember(i["uuid"], i["role"], i["time"], self.key))
        
        return self.membersList

class GuildLBGuild(GuildStats):
    def __init__(self, infoData, membersData, placing, key=""):
        self.placing = placing
        GuildStats.__init__(self, infoData, membersData, key)

class GuildMember:
    def __init__(self, uuid, role, joinTime, key):
        self.uuid = uuid
        self.ign = getUserInfo.getIGN(uuid)
        self.role = role
        self.joinTimeRaw = joinTime
        self.joinTIme = datetime.fromtimestamp(self.joinTimeRaw).strftime("%I:%M %p on %B %d, %Y")
        self.key = key
    
    def get_stats(self):
        info = requests.get(f"{VOXYLBASE}{VOXYLINFO.format(self.uuid, VOXYLAPI.format(self.key))}").json()
        over = requests.get(f"{VOXYLBASE}{VOXYLOVERALL.format(self.uuid, VOXYLAPI.format(self.key))}").json()
        game = requests.get(f"{VOXYLBASE}{VOXYLGAME.format(self.uuid, VOXYLAPI.format(self.key))}").json()

        if not info["success"] or not over["success"] or not game["success"]:
            return None
        
        return PlayerStats(gen=info, over=over, game=game["stats"])

def get_members(memberList):
    members = {"owner": [], "admin": [], "moderator": [], "member": []}

    for i in memberList:
        members[i["role"].lower()].append(i["uuid"])
    
    return members

class GuildNotFound(Exception):
    pass

class GuildMemberNotFound(Exception):
    pass


from datetime import date, datetime

from .constants import *

class PlayerStats():
    def __init__(self, uuid=DEFAULTUUID ,gen=DEFAULTINFO, over=DEFAULTOVER, game=DEFAULTGAME):
        total = calculate_total(game)
        self.uuid = uuid
        self.lastLoginName = gen["lastLoginName"]
        self.lastLoginTimeRaw = gen["lastLoginTime"]
        self.lastLoginTime = datetime.fromtimestamp(self.lastLoginTimeRaw).strftime("%I:%M %p on %B %d, %Y")
        self.role = gen["role"]
        self.level = over["level"]
        self.weightedwins = over["weightedwins"]
        self.xp = over["exp"]
        self.wins = total["wins"]
        self.finals = total["finals"]
        self.kills = total["kills"]
        self.beds = total["beds"]

    def generalInfo(self):
        print(f'''General Info:
Last Login Name: {self.lastLoginName}
Last Login Time: {self.lastLoginTime}
Rank: {self.role}''')

    def generalStats(self):
        print(f'''General Stats:
Level: {self.level}
XP: {self.xp}
Weighted Wins: {self.weightedwins}''')

    def overallGameStats(self):
        print(f'''Overall Game Stats:
Total Wins: {self.wins}
Total Finals: {self.finals}
Total Kills: {self.kills}
Total Beds Destroyed: {self.beds}''')

class WeightedLevelLBPlayer(PlayerStats):
    def __init__(self, placing, uuid=DEFAULTUUID, gen=DEFAULTINFO, over=DEFAULTOVER, game=DEFAULTGAME):
        super().__init__()
        self.placing = placing
        self.uuid = uuid
        total = calculate_total(game)
        self.lastLoginName = gen["lastLoginName"]
        self.lastLoginTimeRaw = gen["lastLoginTime"]
        self.lastLoginTime = datetime.fromtimestamp(self.lastLoginTimeRaw).strftime("%I:%M %p on %B %d, %Y")
        self.role = gen["role"]
        self.level = over["level"]
        self.weightedwins = over["weightedwins"]
        self.xp = over["exp"]
        self.wins = total["wins"]
        self.finals = total["finals"]
        self.kills = total["kills"]
        self.beds = total["beds"]

class TechniqueLBPlayer(PlayerStats):
    def __init__(self, placing, time, subtime, uuid=DEFAULTUUID, gen=DEFAULTINFO, over=DEFAULTOVER, game=DEFAULTGAME):
        super().__init__()
        self.placing = placing
        self.uuid = uuid
        self.time = time
        self.subtime = subtime
        total = calculate_total(game)
        self.lastLoginName = gen["lastLoginName"]
        self.lastLoginTimeRaw = gen["lastLoginTime"]
        self.lastLoginTime = datetime.fromtimestamp(self.lastLoginTimeRaw).strftime("%I:%M %p on %B %d, %Y")
        self.role = gen["role"]
        self.level = over["level"]
        self.weightedwins = over["weightedwins"]
        self.xp = over["exp"]
        self.wins = total["wins"]
        self.finals = total["finals"]
        self.kills = total["kills"]
        self.beds = total["beds"]

def calculate_total(data):
    wins = 0
    finals = 0
    kills = 0
    beds = 0
    for i in data:
        try:
            wins += data[i]["wins"]
        
        except:
            pass

        try:
            finals += data[i]["finals"]
        
        except:
            pass

        try:
            kills += data[i]["kills"]
        
        except:
            pass

        try:
            beds += data[i]["beds"]
        
        except:
            pass
    
    return {"wins": wins, "finals": finals, "kills": kills, "beds": beds}


import requests
from voxyl.constants import *

def getUUID(ign):
    mojdata = requests.get(f"{MOJUUID}{ign}")
    
    if mojdata.status_code != 200:
        raise UnknownPlayer(f"Player not found: '{ign}'")

    mojdata = mojdata.json()
    return mojdata["id"]

def getIGN(uuid):
    mojdata = requests.get(f"{MOJIGN}{uuid}")
    
    if mojdata.status_code != 200:
        raise UnknownPlayer(f"Player not found: '{uuid}'")

    mojdata = mojdata.json()
    return mojdata["name"]

def formatUUID(uuid):
    uuidFormated = uuid[0:8] + "-" + uuid[8:12] + "-" + uuid[12:16] + "-" + uuid[16:20] + "-" + uuid[20:]
    return uuidFormated

class UnknownPlayer(Exception):
    pass


import requests
from voxyl.guildstats import GuildLB, GuildStats
from voxyl.leaderboardstats import LevelWeightedLB, TechniqueLB

from voxyl.playerstats import PlayerStats
from voxyl.utils import *

from .constants import *

class Client():
    def __init__(self, key):
        r = requests.get(VOXYLBASE + VOXYLINFO.format("7dc5566d-4c2c-40dd-891c-4717d8f44713", VOXYLAPI.format(key)))

        if r.status_code != 200:
            raise InvalidAPIKey(f"Invalid API Key: " + key)
        
        self.key = key
    
    def set_key(self, key):        
        r = requests.get(VOXYLBASE + VOXYLINFO.format("7dc5566d-4c2c-40dd-891c-4717d8f44713", VOXYLAPI.format(key)))

        if r.status_code != 200:
            raise InvalidAPIKey(f"Invalid API Key: " + key)
        
        self.key = key
        return True

    def show_key(self):
        return self.key

    def remove_key(self):
        try:
            self.key = ""
            return True
        
        except Exception as e:
            raise e

    def get_player(self, uuid=None, ign=None):
        if uuid != None:
            if uuid.find("-") == -1:
                uuidFormated = getUserInfo.formatUUID(uuid)
            
            else:
                uuidFormated = uuid
        
        elif ign != None:
            uuidFormated = getUserInfo.formatUUID(getUserInfo.getUUID(ign))

        info = requests.get(f"{VOXYLBASE}{VOXYLINFO.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
        over = requests.get(f"{VOXYLBASE}{VOXYLOVERALL.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
        game = requests.get(f"{VOXYLBASE}{VOXYLGAME.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
       
        if not info["success"] or not over["success"] or not game["success"]:
            return None
        
        return PlayerStats(uuid=uuidFormated, gen=info, over=over, game=game["stats"])
    
    def get_guild(self, tag):
        guildInfo = requests.get(f"{VOXYLBASE}{VOXYLGUILDINFO.format(tag, VOXYLAPI.format(self.key))}").json()
        guildMembers = requests.get(f"{VOXYLBASE}{VOXYLGUILDMEMBERS.format(tag, VOXYLAPI.format(self.key))}").json()

        if not guildInfo["success"] or not guildMembers["success"]:
            return None
        
        return GuildStats(infoData=guildInfo, membersData=guildMembers["members"], key=self.key)
    
    def get_guild_LB(self, num: int = 10):
        guildLB = requests.get(f"{VOXYLBASE}{VOXYLGUILDTOP.format(VOXYLAPI.format(self.key), num)}").json()

        if not guildLB["success"]:
            return None
        
        return GuildLB(guildLBInfo=guildLB["guilds"], num=num, key=self.key)
    
    def getLB(self, type: str = "level", num: int = 10, technique: str = ""):
        teststr = "7dc5566d-4c2c-40dd-891c-4717d8f44713"
        r = requests.get(f"{VOXYLBASE}{VOXYLINFO.format(teststr, VOXYLAPI.format(self.key))}").json()
        
        if not r["success"]:
            return None

        if type.lower() != "technique":
            lbInfo = requests.get(f"{VOXYLBASE}{VOXYLLB[type.lower()].format(VOXYLAPI.format(self.key), num)}").json()["players"]
            return LevelWeightedLB(lbInfo, key=self.key)

        elif type.lower() == "technique":
            lbInfo = requests.get(f"{VOXYLBASE}{VOXYLLB[type.lower()].format(VOXYLAPI.format(self.key), technique)}").json()["guilds"]
            return TechniqueLB(lbInfo, key=self.key)
        
        else:
            raise InvalidLeaderBoard(f"Leaderboard '{type}' does not exist")

class InvalidAPIKey(Exception):
    pass

class InvalidLeaderBoard(Exception):
    pass
    


MOJBASE = "https://api.ashcon.app/"
MOJUUID = "https://api.mojang.com/users/profiles/minecraft/"
MOJHIST = "user/profiles/{0}/names"
MOJIGN = "https://sessionserver.mojang.com/session/minecraft/profile/"

VOXYLAPIKEY = ""
VOXYLAPI = "?api={0}"
VOXYLBASE = "https://api.voxyl.net/"
VOXYLINFO = "player/info/{0}{1}"
VOXYLOVERALL = "player/stats/overall/{0}{1}"
VOXYLGAME = "player/stats/game/{0}{1}"
VOXYLGUILDINFO = "guild/info/{0}{1}"
VOXYLGUILDMEMBERS = "guild/members/{0}{1}"
VOXYLGUILDTOP = "guild/top{0}&num={1}"
VOXYLANNOUNCEMENTS = "announcement/all{0}"
VOXYLLB = {"level": "leaderboard/normal{0}&num={1}&type=level", "weighted": "leaderboard/normal{0}&num={1}&type=weightedwins", "weightedwins": "leaderboard/normal{0}&num={1}&type=weightedwins", "technique": "leaderboard/technique{0}&technique={1}"}

DEFAULTINFO = {"success": True, "lastLoginName": "", "lastLoginTime": 0, "role": ""}
DEFAULTOVER = {'success': True, 'level': 0, 'exp': 0, 'weightedwins': 0}
DEFAULTGAME = {'bridgesDouble': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'pearlFightSingle': {'kills': '', 'wins': ''}, 'resourceOldSingle': {'kills': ''}, 'bedwarsMegaSolo': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'rankedFoursPractice': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'bowFightSingle': {'kills': '', 'wins': ''}, 'resourceOldDouble': {'kills': '', 'wins': ''}, 'bedwarsLateSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'voidDouble': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'rankedFours': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'groundDouble': {'finals': '', 'kills': '', 'wins': ''}, 'bedRushSingle': {'kills': '', 'wins': ''}, 'flatFightDouble': {'kills': '', 'wins': ''}, 'compBridgeSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'ladderFightSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'ladderFightDouble': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'voidSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'resourceDouble': {'kills': '', 'wins': ''}, 'obstacleSingle': {'wins': ''}, 'bridgesSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'bowFightDouble': {'kills': '', 'wins': ''}, 'miniwarsDouble': {'finals': '', 'kills': '', 'wins': ''}, 'resourceSingle': {'kills': '', 'wins': ''}, 'pearlFightDouble': {'kills': '', 'wins': ''}, 'bedwarsLateDouble': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'sumo': {'kills': '', 'wins': ''}, 'bedRushDouble': {'kills': '', 'wins': ''}, 'partyGames': {'wins': ''}, 'bedwarsEightSolo': {'finals': '', 'kills': ''}, 'bedwarsNormalSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'bedwarsMegaDouble': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'stickFightSingle': {'kills': '', 'wins': ''}, 'flatFightSingle': {'kills': '', 'wins': ''}, 'miniwarsSolo': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'groundSingle': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'betaSumo': {'kills': '', 'wins': ''}, 'bedwarsNormalDouble': {'beds': '', 'finals': '', 'kills': '', 'wins': ''}, 'stickFightDouble': {'kills': '', 'wins': ''}, 'bedwalls': {'beds': '', 'finals': '', 'kills': ''}}
DEFAULTUUID = "7dc5566d-4c2c-40dd-891c-4717d8f44713"


import requests
import json
from voxyl.playerstats import WeightedLevelLBPlayer, TechniqueLBPlayer

from voxyl.utils import getUserInfo
from .constants import *

class LevelWeightedLB:
    def __init__(self, lbInfo, key=""):
        self.lbInfo = lbInfo
        self.key = key
    
    def get_player(self, ign):
        member = getUserInfo.formatUUID(getUserInfo.getUUID(ign))

        for i in self.lbInfo:
            if i["uuid"] == member:
                uuidFormated = i["uuid"]
                info = requests.get(f"{VOXYLBASE}{VOXYLINFO.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
                over = requests.get(f"{VOXYLBASE}{VOXYLOVERALL.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
                game = requests.get(f"{VOXYLBASE}{VOXYLGAME.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
            
                if not info["success"] or not over["success"] or not game["success"]:
                    return None

                return WeightedLevelLBPlayer(uuid=member, gen=info, over=over, game=game["stats"], placing=i["position"])

        raise LeaderboardPlayerNotFound(f"{ign} not found in leaderboards")

class TechniqueLB:
    def __init__(self, lbInfo, key=""):
        self.lbInfo = lbInfo
        self.key = key
    
    def get_player(self, ign):
        member = getUserInfo.formatUUID(getUserInfo.getUUID(ign))

        for i in self.lbInfo:
            if i["uuid"] == member:
                uuidFormated = i["uuid"]
                info = requests.get(f"{VOXYLBASE}{VOXYLINFO.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
                over = requests.get(f"{VOXYLBASE}{VOXYLOVERALL.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
                game = requests.get(f"{VOXYLBASE}{VOXYLGAME.format(uuidFormated, VOXYLAPI.format(self.key))}").json()
            
                if not info["success"] or not over["success"] or not game["success"]:
                    return None

                return TechniqueLBPlayer(uuid=member, gen=info, over=over, game=game["stats"], placing=i["position"], time=i["time"], subtime=i["submittime"])

        raise LeaderboardPlayerNotFound(f"{ign} not found in leaderboards")

class LeaderboardPlayerNotFound(Exception):
    pass


"""Voxyl API Wrapper

A  simple python wrapper for the Voxyl API

"""


__title__ = 'voxapiwrapper'
__author__ = '_lightninq'
__license__ = 'MIT'
__copyright__ = 'Copyright 2022 BWP-Stats'
__version__ = '1.0.0'


from .guildstats import GuildStats
from .leaderboardstats import LevelWeightedLB, TechniqueLB
from .playerstats import PlayerStats
from .utils import *
from .client import Client



from setuptools import setup, find_packages
import pathlib


HERE = pathlib.Path(__file__).parent
with open(f"{HERE}/README.md") as f:
    README = f.read()



setup(
    name='voxylstats',
    packages=find_packages(),
    version='0.2.8',
    description='A simple python wrapper for the Voxyl API',
    long_description=README,
    author='_lightninq & firestarad',
    license='MIT'
)

